"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const babel_target_1 = require("./babel.target");
const PLUGIN_NAME = 'NamedLazyChunksPlugin';
/**
 * Gives names to lazy chunks (lazy routes) so their assets have recognizable names instead of just numbers.
 */
class NamedLazyChunksPlugin {
    getNameFromOrigins(chunk) {
        const nameInfo = [...chunk.groupsIterable].reduce((result, group) => {
            if (!group.origins) {
                return;
            }
            group.origins.forEach((origin) => {
                if (!origin.request || !origin.request.match(/\.ngfactory$/)) {
                    return;
                }
                const cleanedName = origin.request.replace(/\.ngfactory$/, '');
                const nameStart = cleanedName.lastIndexOf('/') + 1;
                const originName = cleanedName.substring(nameStart);
                if (!result.origins.includes(originName)) {
                    result.origins.push(originName);
                }
                if (!result.babelTarget) {
                    result.babelTarget = babel_target_1.BabelTarget.findTarget(origin.module);
                }
            });
            return result;
        }, { origins: [] });
        const name = nameInfo.origins.join('~');
        return nameInfo.babelTarget.tagAssetsWithKey ? `${name}.${nameInfo.babelTarget.key}` : name;
    }
    apply(compiler) {
        compiler.hooks.compilation.tap(PLUGIN_NAME, (compilation) => {
            if (compilation.name) {
                return;
            }
            compilation.hooks.beforeChunkIds.tap(PLUGIN_NAME, (chunks) => {
                chunks.forEach(chunk => {
                    if (chunk.id || chunk.name) {
                        return;
                    }
                    const isVendorsChunk = chunk.chunkReason === 'split chunk (cache group: vendors)';
                    chunk.id = `${isVendorsChunk ? 'vendors~' : ''}${this.getNameFromOrigins(chunk)}`;
                });
            });
        });
    }
}
exports.NamedLazyChunksPlugin = NamedLazyChunksPlugin;
//# sourceMappingURL=named.lazy.chunks.plugin.js.map